# -*- coding: utf-8 -*-
from __future__ import unicode_literals


def get_node_text(node):
    """
    Return the text content of an xml.dom Element Node.

    If node does not have content, this function return an empty string.
    """
    text = ''

    node.normalize()
    if node.firstChild and node.firstChild.data:
        text = node.firstChild.data.strip()

    return text


def get_urlpath_part(urlpath):
    """
    Return a path without url fragment (something like `#frag` at the end).

    This function allow to use path from references and NCX file to read
    item from Manifest with a correct href (without losing the fragment part).

    eg.:

        url = 'text/chapter1.xhtml#part2'
        href, fragment = get_urlpath_part(url)
        print href # 'text/chapter1.xhtml'
        print fragment # '#part2'
    """
    href = urlpath
    fragment = None
    if urlpath.count('#'):
        href, fragment = urlpath.split('#')
    return (href, fragment)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


import os
import unittest
import epub


from shutil import copy


TEST_XHTML_MIMETYPE = 'application/xhtml+xml'


class TestFunction(unittest.TestCase):
    epub_path = '_data/test.epub'

    def test_version(self):
        self.assertEqual(epub.__version__, '0.5.0')

    def test_open(self):
        test_path = os.path.join(os.path.dirname(__file__), self.epub_path)
        book = epub.open(test_path)

        self.assertEqual(book.opf_path, 'OEBPS/content.opf')
        self.assertEqual(book.content_path, 'OEBPS')
        self.assertEqual(book.opf.metadata.languages, ['en'])
        self.assertEqual(book.opf.metadata.titles, [('Testing Epub', '')])
        self.assertEqual(len(book.opf.manifest), 7)

        for key, item in book.opf.manifest.items():
            self.assertEqual(item.identifier, key)
            self.assertIsInstance(item, epub.opf.ManifestItem)

        with epub.open(test_path) as with_book:
            self.assertEqual(with_book.opf.metadata.languages, ['en'])
            self.assertEqual(with_book.opf.metadata.titles,
                             [('Testing Epub', '')])
            self.assertEqual(len(with_book.opf.manifest), 7)
            for key, item in with_book.opf.manifest.items():
                self.assertEqual(item.identifier, key)
                self.assertIsInstance(item, epub.opf.ManifestItem)


class TestFunctionWriteMode(unittest.TestCase):

    epub_path = '_data/write/test.epub'
    xhtml_item_path = '_data/write/add_item.xhtml'

    def _subtest_add_item(self, book):
        """Test very basic add_item feature.

        The purpose of this subtest is to test a very few basic featureset
        that are common to both [w]rite and [a]ppend mode.
        """
        # Test if we realy are in write/append mode
        self.assertIn(book.mode, ['a', 'w'])

        filename = os.path.join(os.path.dirname(__file__),
                                self.xhtml_item_path)
        manifest_item = epub.opf.ManifestItem(identifier='AddItem0001',
                                              href='Text/add_item.xhtml',
                                              media_type=TEST_XHTML_MIMETYPE)
        book.add_item(filename, manifest_item, True)

        with open(filename, 'r') as f:
            expected_data = f.read()
        result_data = book.read_item(manifest_item).decode('utf-8')

        self.assertIn(manifest_item, book.opf.manifest)
        self.assertEqual(result_data, expected_data)
        self.assertIn((manifest_item.identifier, True),
                      book.opf.spine.itemrefs)


class TestFunctionWriteModeNew(TestFunctionWriteMode):

    def setUp(self):
        self._clean_files()

    def tearDown(self):
        self._clean_files()

    def _clean_files(self):
        filename = os.path.join(os.path.dirname(__file__), self.epub_path)
        if os.path.isfile(filename):
            os.remove(filename)

    def test_open(self):
        working_copy_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_path)

        # test that the file is correctly open in [a]ppend mode.
        book = epub.open(working_copy_filename, 'w')
        self.assertEqual(book.opf_path, 'OEBPS/content.opf')
        self.assertEqual(book.content_path, 'OEBPS')
        self.assertEqual(book.opf.metadata.languages, [])
        self.assertEqual(book.opf.metadata.titles, [])
        self.assertEqual(len(book.opf.manifest), 1)
        # 1 meta file: mimetype
        # 0 content file
        self.assertEqual(len(book.namelist()), 1)

        self._subtest_add_item(book)
        book.close()


class TestFunctionWriteModeAppend(TestFunctionWriteMode):

    epub_source = '_data/write/source.epub'
    epub_empty = '_data/write/test_empty.epub'

    def setUp(self):
        """
        Create a copy of source epub.
        """
        source_filename = os.path.join(os.path.dirname(__file__),
                                       self.epub_source)
        working_copy_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_path)
        copy(source_filename, working_copy_filename)

        self._clean_files()

    def tearDown(self):
        """
        Destroy copy of source epub.
        """
        working_copy_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_path)
        if os.path.isfile(working_copy_filename):
            os.remove(working_copy_filename)

        self._clean_files()

    def _clean_files(self):
        working_empty_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_empty)
        if os.path.isfile(working_empty_filename):
            os.remove(working_empty_filename)

    def test_open(self):
        working_copy_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_path)

        # test that the file is correctly open in [a]ppend mode.
        book = epub.open(working_copy_filename, 'a')
        self.assertEqual(book.opf_path, 'OEBPS/content.opf')
        self.assertEqual(book.content_path, 'OEBPS')
        self.assertEqual(book.opf.metadata.languages, ['fr'])
        self.assertEqual(book.opf.metadata.titles, [('Il était une fois...', '')])
        self.assertEqual(len(book.opf.manifest), 2)
        # 4 meta files: mimetype, container.xml, content.opf, toc.ncx,
        # 1 content file: Section0001.xhtml
        self.assertEqual(len(book.namelist()), 5)

        self._subtest_add_item(book)
        book.close()

    def test_open_new(self):
        working_copy_filename = os.path.join(os.path.dirname(__file__),
                                             self.epub_empty)

        # test that the file is correctly open in [a]ppend mode.
        book = epub.open(working_copy_filename, 'a')
        self.assertEqual(book.opf_path, 'OEBPS/content.opf')
        self.assertEqual(book.content_path, 'OEBPS')
        self.assertEqual(book.opf.metadata.languages, [])
        self.assertEqual(book.opf.metadata.titles, [])
        self.assertEqual(len(book.opf.manifest), 1)
        # 1 meta file: mimetype
        # 0 content file
        self.assertEqual(len(book.namelist()), 1)

        self._subtest_add_item(book)
        book.close()


class TestEpubFile(unittest.TestCase):
    """Test class for epub.EpubFile class"""

    epub_path = '_data/test.epub'
    extracted_dir = '_data'
    extracted_name = 'OEBPS/Text/Section0002.xhtml'

    def setUp(self):
        test_path = os.path.join(os.path.dirname(__file__), self.epub_path)
        self.epub_file = epub.open(test_path)

    def tearDown(self):
        self.epub_file.close()

        extracted_filename = os.path.join(os.getcwd(),
                                          self.extracted_name)
        if os.path.isfile(extracted_filename):
            os.remove(extracted_filename)

        extracted_filename = os.path.join(os.path.dirname(__file__),
                                          self.extracted_dir,
                                          self.extracted_name)
        if os.path.isfile(extracted_filename):
            os.remove(extracted_filename)

    def test_extract_item(self):
        item = self.epub_file.get_item('Section0002.xhtml')

        self.epub_file.extract_item(item)
        extracted_filename = os.path.join(os.getcwd(),
                                          self.extracted_name)
        self.assertTrue(os.path.isfile(extracted_filename))

    def test_extract_item_to_path(self):
        item = self.epub_file.get_item('Section0002.xhtml')
        extracted_dir = os.path.join(os.path.dirname(__file__),
                                     self.extracted_dir)

        self.epub_file.extract_item(item, to_path=extracted_dir)
        extracted_filename = os.path.join(extracted_dir,
                                          self.extracted_name)
        self.assertTrue(os.path.isfile(extracted_filename))

    def test_get_item(self):
        """Check EpubFile.get_item() return an EpubManifestItem by its id"""
        item = self.epub_file.get_item('Section0002.xhtml')
        self.assertIsInstance(item, epub.opf.ManifestItem,
                              'L\'item retourné doit être un objet de type <epub.opf.ManifestItem>')
        self.assertEqual(item.identifier, 'Section0002.xhtml', 'id attendu incorrect.')
        self.assertEqual(item.href, 'Text/Section0002.xhtml',
                         'href attendu incorrect.')

        self.assertEqual(self.epub_file.get_item('BadId'), None)

    def test_get_item_by_ref(self):
        """Check EpubFile.get_item() return an EpubManifestItem by its href"""
        item = self.epub_file.get_item_by_href('Text/Section0002.xhtml')
        self.assertIsInstance(item, epub.opf.ManifestItem,
                              'L\'item retourné doit être un objet de type <epub.opf.ManifestItem>')
        self.assertEqual(item.identifier, 'Section0002.xhtml', 'id attendu incorrect.')
        self.assertEqual(item.href, 'Text/Section0002.xhtml',
                         'href attendu incorrect.')

        self.assertEqual(self.epub_file.get_item_by_href('BadHref'), None)

        # Change only Id, so there is 2 item with the same href attribute
        item.identifier = 'CopyOfSection0002.xhtml'
        self.epub_file.opf.manifest.append(item)

        with self.assertRaises(LookupError):
            self.epub_file.get_item_by_href(item.href)

    def test_add_item_fail(self):
        """
        When open in read-only mode, add_item must fail.
        """
        with self.assertRaises(IOError):
            filename = '_data/write/add_item.xhtml'
            manifest_item = epub.opf.ManifestItem(identifier='AddItem0001',
                                                  href='Text/add_item.xhtml',
                                                  media_type=TEST_XHTML_MIMETYPE)
            self.epub_file.add_item(filename, manifest_item)

        self.epub_file.close()
        with self.assertRaises(RuntimeError):
            filename = '_data/write/add_item.xhtml'
            manifest_item = epub.opf.ManifestItem(identifier='AddItem0001',
                                                  href='Text/add_item.xhtml',
                                                  media_type=TEST_XHTML_MIMETYPE)
            self.epub_file.add_item(filename, manifest_item)


class TestBook(unittest.TestCase):

    epub_path = '_data/test.epub'

    def setUp(self):
        test_path = os.path.join(os.path.dirname(__file__), self.epub_path)
        self.epub_file = epub.open(test_path)

    def tearDown(self):
        self.epub_file.close()

    def test_chapters(self):
        book = epub.Book(self.epub_file)
        self.assertEquals(len(book.chapters), 6)

        for chapter in book.chapters:
            self.assertIsNotNone(chapter)

    def test_extra_chapters(self):
        book = epub.Book(self.epub_file)
        self.assertEquals(len(list(book.extra_chapters)), 0)

    def test_metadata_creators(self):
        name = 'Johnson Cave'
        role = 'aut'
        file_as = 'Cave, Johnson'
        self.epub_file.opf.metadata.add_creator(name, role, file_as)
        book = epub.Book(self.epub_file)
        self.assertIn((name, role, file_as), book.creators)

    def test_metadata_description(self):
        description = self.epub_file.opf.metadata.description
        book = epub.Book(self.epub_file)
        self.assertEquals(book.description, description)

    def test_metadata_isbn(self):
        isbn = '7814-54654-4354-43545'
        self.epub_file.opf.metadata.add_identifier(isbn, 'ID_ISBN', 'isbn')
        book = epub.Book(self.epub_file)
        self.assertEquals(book.isbn, isbn)

    def test_metadata_publisher(self):
        publisher = 'TEST_PUBLISHER'
        self.epub_file.opf.metadata.publisher = publisher
        book = epub.Book(self.epub_file)
        self.assertEquals(book.publisher, publisher)

    def test_metadata_contributors(self):
        name = 'Johnson Cave'
        role = 'oth'
        file_as = 'Cave, Johnson'
        self.epub_file.opf.metadata.add_contributor(name, role, file_as)
        book = epub.Book(self.epub_file)
        self.assertIn((name, role, file_as), book.contributors)

    def test_dates(self):
        date = '2032-08-04'
        event = 'publication'
        self.epub_file.opf.metadata.add_date(date, event)
        book = epub.Book(self.epub_file)
        self.assertIn((date, event), book.dates)

    def test_dc_type(self):
        self.epub_file.opf.metadata.dc_type = 'test_type'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.dc_type, 'test_type')

    def test_format(self):
        self.epub_file.opf.metadata.format = 'test_format'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.format, 'test_format')

    def test_identifiers(self):
        book = epub.Book(self.epub_file)
        self.assertEquals(book.identifiers,
                          [('urn:uuid:477d1a82-a70d-4ee5-a0ff-0dddc60fd2bb',
                            'BookId',
                            'UUID')])

    def test_source(self):
        self.epub_file.opf.metadata.source = 'test_source'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.source, 'test_source')

    def test_languages(self):
        lang = 'test_lang'
        self.epub_file.opf.metadata.add_language(lang)
        book = epub.Book(self.epub_file)
        self.assertIn(lang, book.languages)

    def test_relation(self):
        self.epub_file.opf.metadata.relation = 'test_relation'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.relation, 'test_relation')

    def test_coverage(self):
        self.epub_file.opf.metadata.coverage = 'test_coverage'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.coverage, 'test_coverage')

    def test_right(self):
        self.epub_file.opf.metadata.right = 'test_right'
        book = epub.Book(self.epub_file)
        self.assertEquals(book.right, 'test_right')

    def test_metas(self):
        name = 'meta_name'
        content = 'meta_content'
        self.epub_file.opf.metadata.add_meta(name, content)
        book = epub.Book(self.epub_file)
        self.assertIn((name, content), book.metas)

    def test_subjects(self):
        title = 'New test subject'
        self.epub_file.opf.metadata.add_subject(title)
        book = epub.Book(self.epub_file)
        self.assertIn(title, book.subjects)

    def test_titles(self):
        title = 'New test title'
        lang = 'fra_alt'
        self.epub_file.opf.metadata.add_title(title, lang)
        book = epub.Book(self.epub_file)
        self.assertIn((title, lang), book.titles)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


import unittest
import epub


from xml.dom import minidom


class TestFunction(unittest.TestCase):
    epub_path = '_data/test.epub'

    def test_get_node_text(self):
        expected_result = 'Has text.'
        xml_text = """<someNode>%s</someNode>""" % expected_result
        xml_node = minidom.parseString(xml_text).documentElement

        self.assertEquals(epub.utils.get_node_text(xml_node), expected_result)

        xml_text = """<someEmptyNode></someEmptyNode>"""
        xml_node = minidom.parseString(xml_text).documentElement

        self.assertEquals(epub.utils.get_node_text(xml_node), '')

    def test_get_urlpath_part(self):
        expected_href = 'path/to/file.html'
        expected_fragment = 'withfragment'
        url = '%s#%s' % (expected_href, expected_fragment)

        href, fragment = epub.utils.get_urlpath_part(url)
        self.assertEquals(href, expected_href)
        self.assertEquals(fragment, expected_fragment)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
Library to open and read files in the epub version 2.
"""


__author__ = 'Florian Strzelecki <florian.strzelecki@gmail.com>'
__version__ = '0.5.0'
__all__ = ['opf', 'ncx', 'utils']


import os
import uuid
import zipfile


from xml.dom import minidom


from epub import ncx, opf, utils


MIMETYPE_EPUB = 'application/epub+zip'
MIMETYPE_OPF = 'application/oebps-package+xml'
MIMETYPE_NCX = 'application/x-dtbncx+xml'

DEFAULT_OPF_PATH = 'OEBPS/content.opf'
DEFAULT_NCX_PATH = 'toc.ncx'


def open(filename, mode=None):
    """Open an epub file and return an EpubFile object"""
    import warnings
    warnings.warn('Function `epub.open` is deprecated since 0.5.0.',
                  DeprecationWarning)
    return open_epub(filename, mode)


def open_epub(filename, mode=None):
    return EpubFile(filename, mode)


class BadEpubFile(zipfile.BadZipfile):
    pass


class EpubFile(zipfile.ZipFile):
    """
    Represent an epub zip file, as described in version 2.0.1 of epub spec.

    This class allow an access throught a low-level API to the epub real file.
    It extends zipfile.ZipFile class and modify only a little some of its
    behavior.

    See http://idpf.org/epub/201 for more information about Epub 2.0.1.
    """

    @property
    def content_path(self):
        """
        Return the content path, ie, the path relative to OPF file.

        If OPF file is located in `OEBPS/content.opf`, then `content_path` is
        equal to `OEBPS`.
        """
        return os.path.dirname(self.opf_path)

    def __init__(self, filename, mode=None):
        """
        Open the Epub zip file with mode read "r", write "w" or append "a".
        """
        mode = mode or 'r'
        zipfile.ZipFile.__init__(self, filename, mode)

        if self.mode == 'r':
            self._init_read()
        elif self.mode == 'w':
            self._init_new()
        elif self.mode == 'a':
            if len(self.namelist()) == 0:
                self._init_new()
            else:
                self._init_read()

    def _init_new(self):
        """
        Build an empty epub archive.
        """
        # Write mimetype file: 'application/epub+zip'
        self.writestr('mimetype', MIMETYPE_EPUB)
        # Default path for opf
        self.opf_path = DEFAULT_OPF_PATH
        # Uid & Uid's id
        uid_id = 'BookId'
        self.uid = '%s' % uuid.uuid4()
        # Create metadata, manifest, and spine, as minimalist as possible
        metadata = opf.Metadata()
        metadata.add_identifier(self.uid, uid_id, 'uid')
        manifest = opf.Manifest()
        manifest.add_item('ncx', 'toc.ncx', MIMETYPE_NCX)
        spine = opf.Spine('ncx')
        # Create Opf object
        self.opf = opf.Opf(uid_id=uid_id,
                           metadata=metadata, manifest=manifest, spine=spine)
        # Create Ncx object
        self.toc = ncx.Ncx()
        self.toc.uid = self.uid

    def _init_read(self):
        # Read container.xml to get OPF xml file path
        xmlstring = self.read('META-INF/container.xml')
        container_xml = minidom.parseString(xmlstring).documentElement

        for e in container_xml.getElementsByTagName('rootfile'):
            if e.getAttribute('media-type') == MIMETYPE_OPF:
                # Only take the first full-path available
                self.opf_path = e.getAttribute('full-path')
                break

        # Read OPF xml file
        xml_string = self.read(self.opf_path)
        self.opf = opf.parse_opf(xml_string)
        self.uid = [x for x in self.opf.metadata.identifiers
                      if x[1] == self.opf.uid_id][0]
        item_toc = self.get_item(self.opf.spine.toc)

        # Inspect NCX toc file
        self.toc = ncx.parse_toc(self.read_item(item_toc))

    def close(self):
        if self.fp is None:
            return
        if self.mode in ('w', 'a'):
            self._write_close()
        zipfile.ZipFile.close(self)

    def _write_close(self):
        """
        Handle writes when closing epub. Both new file mode (w) and append
        file mode (a), some files must be generated: container, OPF, and NCX.
        """
        # Write META-INF/container.xml
        self.writestr('META-INF/container.xml',
                      self._build_container().encode('utf-8'))
        # Write OPF File
        self.writestr(self.opf_path,
                      self.opf.as_xml_document().toxml().encode('utf-8'))
        # Write NCX File
        item_toc = self.get_item(self.opf.spine.toc)
        self.writestr(os.path.join(self.content_path, item_toc.href),
                      self.toc.as_xml_document().toxml().encode('utf-8'))

    def _build_container(self):
        template = """<?xml version="1.0" encoding="UTF-8"?>
    <container version="1.0"
               xmlns="urn:oasis:names:tc:opendocument:xmlns:container">
        <rootfiles>
             <rootfile full-path="%s"
                       media-type="application/oebps-package+xml"/>
        </rootfiles>
    </container>"""
        return template % self.opf_path

    def add_item(self, filename, manifest_item,
                 append_to_spine=False, is_linear=True):
        """
        Add a file to epub. A manifest item must be provide to describe it.
        This function will raise a RuntimeError if epub is already closed. It
        will raise an IOError if epub is open in read-only (`r` mode).

        Optional: you can use `append_to_spine` flag (default=False) to append
        item to spine, and use `is_linear` (default=True) to specify if it is
        linear or not.
        """
        self.check_mode_write()
        self.opf.manifest.append(manifest_item)
        self.write(filename, os.path.join(self.content_path,
                                          manifest_item.href))
        if append_to_spine:
            self.opf.spine.add_itemref(manifest_item.identifier, is_linear)

    def check_mode_write(self):
        """
        Raise error if epub file is not writable.

        Raise RuntimeError if file is already closed.

        Raise IOError if file is opened read-only.
        """
        if not self.fp:
            raise RuntimeError(
                  'Attempt to write to EPUB file that was already closed')

        if self.mode == 'r':
            raise IOError(
                  'Attempt to write to EPUB file that was open as read-only.')

    # extract method is  zipfile.ZipFile.extract(member[, path[, pwd]])

    def extract_item(self, item, to_path=None):
        """
        Extract an item from its href in epub archive to `to_path` location.
        """
        path = item
        if hasattr(item, 'href'):
            path = item.href
        return  self.extract(member=os.path.join(self.content_path, path),
                             path=to_path)

    def get_item(self, identifier):
        """
        Get an item from manifest through its "id" attribute.

        Return an EpubManifestItem if found, else None.
        """
        return self.opf.manifest.get(identifier, None)

    def get_item_by_href(self, href):
        """
        Get an item from manifest through its "href" attribute.

        Return an EpubManifestItem if found, else None.
        """
        l = [x for x in self.opf.manifest.values() if x.href == href]
        size = len(l)
        if size == 1:
            return l[0]
        elif size > 1:
            raise LookupError('Multiple items are found with this href.')
        else:
            return None

    # read method is zipfile.ZipFile.read(path)

    def read_item(self, item):
        """
        Read a file from the epub zipfile container.

        "item" parameter can be the relative path to the opf file or an
        EpubManifestItem object.

        Html fragments are not acceptable : the path must be exactly the same
        as indicated in the opf file.
        """
        path = item
        if hasattr(item, 'href'):
            path = item.href
        return self.read(os.path.join(self.content_path, path))


class Book(object):
    """
    This class is an attempt (work in progress) to expose a simpler object
    model than EpubFile.
    """

    def __init__(self, epub_file):
        self.epub_file = epub_file

    @property
    def creators(self):
        return self.epub_file.opf.metadata.creators

    @property
    def description(self):
        return self.epub_file.opf.metadata.description

    @property
    def isbn(self):
        return self.epub_file.opf.metadata.get_isbn()

    @property
    def publisher(self):
        return self.epub_file.opf.metadata.publisher

    @property
    def contributors(self):
        return self.epub_file.opf.metadata.contributors

    @property
    def dates(self):
        return self.epub_file.opf.metadata.dates

    @property
    def dc_type(self):
        return self.epub_file.opf.metadata.dc_type

    @property
    def format(self):
        return self.epub_file.opf.metadata.format

    @property
    def identifiers(self):
        return self.epub_file.opf.metadata.identifiers

    @property
    def source(self):
        return self.epub_file.opf.metadata.source

    @property
    def languages(self):
        return self.epub_file.opf.metadata.languages

    @property
    def relation(self):
        return self.epub_file.opf.metadata.relation

    @property
    def coverage(self):
        return self.epub_file.opf.metadata.coverage

    @property
    def right(self):
        return self.epub_file.opf.metadata.right

    @property
    def metas(self):
        return self.epub_file.opf.metadata.metas

    @property
    def subjects(self):
        return self.epub_file.opf.metadata.subjects

    @property
    def titles(self):
        return self.epub_file.opf.metadata.titles

    @property
    def chapters(self):
        """
        Return a list of linear chapter from spine.
        """
        return [BookChapter(self, identifier)
                for identifier, linear in self.epub_file.opf.spine.itemrefs
                if linear]

    @property
    def extra_chapters(self):
        """
        Return a list of non-linear chapter from spine.
        """
        return [BookChapter(self, identifier)
                for identifier, linear in self.epub_file.opf.spine.itemrefs
                if not linear]


class BookChapter(object):

    def __init__(self, book, identifier, fragment=None):
        self._book = book
        self._manifest_item = self._book.epub_file.get_item(identifier)
        self._fragment = fragment

    def read(self):
        return self._book.epub_file.read_item(self._manifest_item)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


import os
import unittest


from xml.dom import minidom


import epub


class TestFunction(unittest.TestCase):
    """Test case for all the _parse_* function."""

    ncx_path = '_data/test.ncx'

    def test_parse_toc(self):
        test_path = os.path.join(os.path.dirname(__file__), self.ncx_path)
        doc = minidom.parse(test_path)
        xml_string = doc.toprettyxml()
        toc = epub.ncx.parse_toc(xml_string)

        self.assertEqual(toc.xmlns,
                         'http://www.daisy.org/z3986/2005/ncx/')
        self.assertEqual(toc.version,
                         '2005-1')
        self.assertEqual(toc.lang, 'en-US')
        self.assertEqual(toc.title,
                         'Selections from "Great Pictures, As Seen and Described by Famous Writers"')
        self.assertEqual(toc.authors,
                         ['Esther Singleton',
                          'Test Author'])
        self.assertEqual(toc.uid,
                         'org-example-5059463624137734586')
        # nav map
        self.assertIsInstance(toc.nav_map, epub.ncx.NavMap)
        self.assertEqual(len(toc.nav_map.nav_point), 2,
                         'Il manque des nav_point !')
        # page list
        self.assertIsInstance(toc.page_list, epub.ncx.PageList)
        self.assertEqual(len(toc.page_list.page_target), 2,
                         'Il manque des page_target !')
        # nav list
        self.assertEqual(len(toc.nav_lists), 1,
                         'Il manque des nav_list !')
        self.assertIsInstance(toc.nav_lists[0], epub.ncx.NavList)
        self.assertEqual(len(toc.nav_lists[0].nav_target), 2,
                         'Il manque des page_target !')

    def test_parse_for_text_tag(self):
        """Test function "_parse_for_text_tag"."""

        xml_string = """
        <someTag>
            <text>La balise "text" par défaut.</text>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element)
        self.assertEqual(text, 'La balise "text" par défaut.')

        xml_string = """
        <someTag>
            <otherTag>La balise "otherTag" par le paramètre "name".</otherTag>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element, 'otherTag')
        self.assertEqual(text, 'La balise "otherTag" par le paramètre "name".')

        xml_string = """
        <someTag>
            <text>Mauvaise balise !</text>
            <textOk>Bonne balise !</textOk>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element, 'textOk')
        self.assertEqual(text, 'Bonne balise !')

        xml_string = """
        <someTag>
            <text>Première balise uniquement...</text>
            <text>Pas la seconde !</text>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element)
        self.assertEqual(text, 'Première balise uniquement...')

        xml_string = """
        <someTag>
            <text>Je ne cherche pas "text" !</text>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element, 'noText')
        self.assertEqual(text, '', 'Il ne devrait pas y avoir de résultat !')

        xml_string = """
        <someTag>
            <text>
                Du texte avec espace et retour à la ligne.
            </text>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element)
        self.assertEqual(text, 'Du texte avec espace et retour à la ligne.',
                         'Il ne devrait pas y avoir de différence !')

        xml_string = """
        <someTag>
            <emptyText />
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element, 'emptyText')
        self.assertEqual(text, '')

        xml_string = """
        <someTag>
            <emptyText></emptyText>
        </someTag>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        text = epub.ncx._parse_for_text_tag(xml_element, 'emptyText')
        self.assertEqual(text, '')

    def test_parse_xml_nav_target(self):
        """Test function "_parse_xml_nav_target"."""

        xml_string = """
        <navTarget playOrder="5"
                   id="part1_target-fragment"
                   value="5"
                   class="some_class">
            <navLabel><text>Label de la Target</text></navLabel>
            <content src="Text/part1.xhtml#target-fragment" />
        </navTarget>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_target = epub.ncx._parse_xml_nav_target(xml_element)
        self.assertIsInstance(nav_target, epub.ncx.NavTarget)
        self.assertEqual(nav_target.identifier, 'part1_target-fragment')
        self.assertEqual(nav_target.value, '5')
        self.assertEqual(nav_target.class_name, 'some_class')
        self.assertEqual(nav_target.play_order, '5')
        self.assertEqual(nav_target.labels, [('Label de la Target', '', ''),])
        self.assertEqual(nav_target.src, 'Text/part1.xhtml#target-fragment')

        xml_string = """
        <navTarget>
            <navLabel xml:lang="fr"><text>Label français</text></navLabel>
            <navLabel xml:lang="en"><text>English label</text></navLabel>
            <content src="Text/part1.xhtml#target-fragment" />
        </navTarget>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_target = epub.ncx._parse_xml_nav_target(xml_element)
        self.assertEqual(nav_target.labels, [('Label français', 'fr', ''),
                                             ('English label', 'en', ''), ])

    def test_parse_xml_nav_list(self):
        """Test function "_parse_xml_nav_list"."""

        xml_string = """
        <navList id="navlist-1" class="list_class">
            <navLabel xml:lang="fr"><text>Label français</text></navLabel>
            <navLabel xml:lang="en"><text>English label</text></navLabel>
            <navInfo xml:lang="fr"><text>Info de navigation</text></navInfo>
            <navInfo xml:lang="en"><text>Navigation's info</text></navInfo>
            <navTarget playOrder="1"
                   id="part1_target-fragment"
                   value="Some Value"
                   class="some_class">
                <navLabel><text>Label de la Target 1</text></navLabel>
                <content src="Text/part1.xhtml#target-fragment" />
            </navTarget>
            <navTarget playOrder="2"
                       id="part2_target-fragment"
                       value="Some Value"
                       class="some_class">
                <navLabel><text>Label de la Target 2</text></navLabel>
                <content src="Text/part2.xhtml" />
            </navTarget>
        </navList>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_list = epub.ncx._parse_xml_nav_list(xml_element)
        self.assertIsInstance(nav_list, epub.ncx.NavList)
        self.assertEqual(nav_list.identifier, 'navlist-1')
        self.assertEqual(nav_list.class_name, 'list_class')
        self.assertEqual(nav_list.labels, [('Label français', 'fr', ''),
                                           ('English label', 'en', ''), ])
        self.assertEqual(nav_list.infos, [('Info de navigation', 'fr', ''),
                                          ('Navigation\'s info', 'en', ''), ])
        self.assertEqual(len(nav_list.nav_target), 2)

        for nav_target in nav_list.nav_target:
            self.assertIsInstance(nav_target, epub.ncx.NavTarget)
            test_label = 'Label de la Target %s' % nav_target.play_order
            self.assertEqual(nav_target.labels[0], (test_label, '', ''))

    def test_parse_xml_page_target(self):
        """Test function "_parse_xml_page_target"."""

        xml_string = """
        <pageTarget playOrder="5"
                   id="part1_target-fragment"
                   value="Some Value"
                   type="page_type"
                   class="some_class">
            <navLabel><text>Label de la Target</text></navLabel>
            <content src="Text/part1.xhtml#target-fragment" />
        </pageTarget>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        page_target = epub.ncx._parse_xml_page_target(xml_element)
        self.assertIsInstance(page_target, epub.ncx.PageTarget)
        self.assertEqual(page_target.identifier, 'part1_target-fragment')
        self.assertEqual(page_target.value, 'Some Value')
        self.assertEqual(page_target.target_type, 'page_type')
        self.assertEqual(page_target.class_name, 'some_class')
        self.assertEqual(page_target.play_order, '5')
        self.assertEqual(page_target.labels,
                         [('Label de la Target', '', ''), ])
        self.assertEqual(page_target.src, 'Text/part1.xhtml#target-fragment')

        xml_string = """
        <pageTarget>
            <navLabel xml:lang="fr"><text>Label français</text></navLabel>
            <navLabel xml:lang="en"><text>English label</text></navLabel>
            <content src="Text/part1.xhtml#target-fragment" />
        </pageTarget>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_target = epub.ncx._parse_xml_page_target(xml_element)
        self.assertEqual(nav_target.labels, [('Label français', 'fr', ''),
                                             ('English label', 'en', ''), ])

    def test_parse_xml_page_list(self):
        """Test function "_parse_xml_page_list"."""

        xml_string = """
        <pageList id="pagelist-1"
                 class="page_class">
            <navLabel xml:lang="fr"><text>Label français</text></navLabel>
            <navLabel xml:lang="en"><text>English label</text></navLabel>
            <navInfo xml:lang="fr"><text>Info de navigation</text></navInfo>
            <navInfo xml:lang="en"><text>Navigation's info</text></navInfo>
            <pageTarget playOrder="1"
                       id="part1_target-fragment"
                       value="Some Value"
                       type="page_type"
                       class="some_class">
                <navLabel xml:lang="fr"><text>Label français 1</text></navLabel>
                <content src="Text/part1.xhtml#target-fragment" />
            </pageTarget>
            <pageTarget playOrder="2"
                       id="part1_target-fragment"
                       value="Some Value"
                       type="page_type"
                       class="some_class">
                <navLabel xml:lang="fr"><text>Label français 2</text></navLabel>
                <content src="Text/part2.xhtml#target-fragment" />
            </pageTarget>
        </pageList>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        page_list = epub.ncx._parse_xml_page_list(xml_element)
        self.assertIsInstance(page_list, epub.ncx.PageList)
        self.assertEqual(page_list.identifier, 'pagelist-1')
        self.assertEqual(page_list.class_name, 'page_class')
        self.assertEqual(page_list.labels, [('Label français', 'fr', ''),
                                            ('English label', 'en', ''), ])
        self.assertEqual(page_list.infos, [('Info de navigation', 'fr', ''),
                                           ('Navigation\'s info', 'en', ''), ])
        self.assertEqual(len(page_list.page_target), 2)

        for page_target in page_list.page_target:
            self.assertIsInstance(page_target, epub.ncx.PageTarget)
            test_label = 'Label français %s' % page_target.play_order
            self.assertEqual(page_target.labels,
                             [(test_label, 'fr', ''),])

    def test_parse_xml_nav_point(self):
        """Test function "_parse_xml_nav_point"."""

        xml_string = """
        <navPoint playOrder="5"
                   id="point5"
                   class="some_class">
            <navLabel xml:lang="fr"><text>Label fr</text></navLabel>
            <navLabel xml:lang="en"><text>Label en</text></navLabel>
            <content src="Text/Point1.xhtml#fragment5" />
            <navPoint playOrder="6" id="point5_1">
                <navLabel xml:lang="fr"><text>Sous-Label 6</text></navLabel>
                <content src="Text/Point1.xhtml#fragment5_1" />
            </navPoint>
            <navPoint playOrder="7" id="point5_2">
                <navLabel xml:lang="fr"><text>Sous-Label 7</text></navLabel>
                <content src="Text/Point1.xhtml#fragment5_2" />
            </navPoint>
        </navPoint>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_point = epub.ncx._parse_xml_nav_point(xml_element)
        self.assertIsInstance(nav_point, epub.ncx.NavPoint)
        self.assertEqual(nav_point.identifier, 'point5')
        self.assertEqual(nav_point.class_name, 'some_class')
        self.assertEqual(nav_point.labels, [('Label fr', 'fr', ''),
                                             ('Label en', 'en', ''),])
        self.assertEqual(nav_point.src, 'Text/Point1.xhtml#fragment5')
        self.assertEqual(len(nav_point.nav_point), 2)
        for child_nav_point in nav_point.nav_point:
            self.assertIsInstance(child_nav_point, epub.ncx.NavPoint)
            test_label = 'Sous-Label %s' % child_nav_point.play_order
            self.assertEqual(child_nav_point.labels,
                             [(test_label, 'fr', ''),])

    def test_parse_xml_nav_map(self):
        xml_string = """
        <navMap id="the_only_one_map">
            <navPoint playOrder="1">
                <navLabel><text>Introduction</text></navLabel>
                <content src="Text/introduction.xhtml" />
            </navPoint>
            <navPoint playOrder="5"
                       id="point5"
                       class="some_class">
                <navLabel xml:lang="fr"><text>Label fr</text></navLabel>
                <navLabel xml:lang="en"><text>Label en</text></navLabel>
                <content src="Text/Point1.xhtml#fragment5" />
                <navPoint playOrder="6" id="point5_1">
                    <navLabel xml:lang="fr"><text>Sous-Label 6</text></navLabel>
                    <content src="Text/Point1.xhtml#fragment5_1" />
                </navPoint>
                <navPoint playOrder="7" id="point5_2">
                    <navLabel xml:lang="fr"><text>Sous-Label 7</text></navLabel>
                    <content src="Text/Point1.xhtml#fragment5_2" />
                </navPoint>
            </navPoint>
        </navMap>
        """
        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_map = epub.ncx._parse_xml_nav_map(xml_element)
        self.assertIsInstance(nav_map, epub.ncx.NavMap)
        self.assertEqual(nav_map.identifier, 'the_only_one_map')
        self.assertEqual(len(nav_map.nav_point), 2)
        self.assertEqual(len(nav_map.nav_point[0].nav_point), 0)
        self.assertEqual(len(nav_map.nav_point[1].nav_point), 2)

    def test_create_xml_element_text(self):
        xml_string = """<text>Some text</text>"""
        xml_element = epub.ncx._create_xml_element_text('Some text')
        self.assertEqual(xml_element.toxml(), xml_string)
        
        xml_string = """<otherText>Some text</otherText>"""
        xml_element = epub.ncx._create_xml_element_text('Some text', 'otherText')
        self.assertEqual(xml_element.toxml(), xml_string)

        xml_string = """<emptyText/>"""
        xml_element = epub.ncx._create_xml_element_text(None, 'emptyText')
        self.assertEqual(xml_element.toxml(), xml_string)

        xml_string = """<emptyText/>"""
        xml_element = epub.ncx._create_xml_element_text('', 'emptyText')
        self.assertEqual(xml_element.toxml(), xml_string)


class TestNavPoint(unittest.TestCase):
    
    def test_as_xml_element(self):
        xml_string = """<navPoint class="some_class" id="point5" playOrder="5">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<content src="Text/Point1.xhtml#fragment5"/>""" + \
            """<navPoint id="point5_1" playOrder="6">""" + \
                """<navLabel xml:lang="fr"><text>Sous-Label 6</text></navLabel>""" + \
                """<content src="Text/Point1.xhtml#fragment5_1"/>""" + \
            """</navPoint>""" + \
            """<navPoint id="point5_2" playOrder="7">""" + \
                """<navLabel xml:lang="fr"><text>Sous-Label 7</text></navLabel>""" + \
                """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
            """</navPoint>""" + \
        """</navPoint>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_point = epub.ncx._parse_xml_nav_point(xml_element)

        self.assertEqual(nav_point.as_xml_element().toxml(),
                         xml_element.toxml())


class TestNavMap(unittest.TestCase):
    
    def test_as_xml_element(self):
        xml_string = """<navMap id="someId">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<navInfo xml:lang="fr" dir="ltr"><text>Label fr</text></navInfo>""" + \
            """<navInfo xml:lang="en" dir="ltr"><text>Label en</text></navInfo>""" + \
            """<navPoint id="point5_2" playOrder="7">""" + \
                """<navLabel xml:lang="fr"><text>Sous-Label 7</text></navLabel>""" + \
                """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
            """</navPoint>""" + \
        """</navMap>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_map = epub.ncx._parse_xml_nav_map(xml_element)

        self.assertEqual(nav_map.as_xml_element().toxml(),
                         xml_element.toxml())


class TestPageTarget(unittest.TestCase):

    def test_as_xml_element(self):
        xml_string = """<pageTarget class="some_class" id="testid" playOrder="5" type="page_type" value="Some Value">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
        """</pageTarget>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        page_target = epub.ncx._parse_xml_page_target(xml_element)

        self.assertEqual(page_target.as_xml_element().toxml(),
                         xml_element.toxml())


class TestPageList(unittest.TestCase):

    def test_as_xml_element(self):
        xml_string = """<pageList id="pagelist-1" class="page_class">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<navInfo xml:lang="fr" dir="ltr"><text>Info de navigation</text></navInfo>""" + \
            """<navInfo xml:lang="en" dir="ltr"><text>Navigation's info</text></navInfo>""" + \
            """<pageTarget class="some_class" id="testid" playOrder="5" type="page_type" value="Some Value">""" + \
                """<navLabel xml:lang="fr"><text>Label fr</text></navLabel>""" + \
                """<navLabel xml:lang="en"><text>Label en</text></navLabel>""" + \
                """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
            """</pageTarget>""" + \
        """</pageList>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        page_list = epub.ncx._parse_xml_page_list(xml_element)

        self.assertEqual(page_list.as_xml_element().toxml(),
                         xml_element.toxml())


class TestNavTarget(unittest.TestCase):

    def test_as_xml_element(self):
        xml_string = """<navTarget class="some_class" id="testid" playOrder="5" value="5">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
        """</navTarget>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_target = epub.ncx._parse_xml_nav_target(xml_element)

        self.assertEqual(nav_target.as_xml_element().toxml(),
                         xml_element.toxml())


class TestNavList(unittest.TestCase):

    def test_as_xml_element(self):
        xml_string = """<navList class="some_class" id="testid">""" + \
            """<navLabel xml:lang="fr" dir="ltr"><text>Label fr</text></navLabel>""" + \
            """<navLabel xml:lang="en" dir="ltr"><text>Label en</text></navLabel>""" + \
            """<navInfo xml:lang="fr" dir="ltr"><text>Info de navigation</text></navInfo>""" + \
            """<navInfo xml:lang="en" dir="ltr"><text>Navigation's info</text></navInfo>""" + \
            """<navTarget class="some_class" id="testid" playOrder="5" value="5">""" + \
                """<navLabel xml:lang="fr"><text>Label fr</text></navLabel>""" + \
                """<navLabel xml:lang="en"><text>Label en</text></navLabel>""" + \
                """<content src="Text/Point1.xhtml#fragment5_2"/>""" + \
            """</navTarget>""" + \
        """</navList>"""

        xml_element = minidom.parseString(xml_string.encode('utf-8')).documentElement
        nav_list = epub.ncx._parse_xml_nav_list(xml_element)

        self.assertEqual(nav_list.as_xml_element().toxml(),
                         xml_element.toxml())


class TestNcx(unittest.TestCase):
    
    def test_as_xml_document(self):
        """Check if ncx.as_xml_document reproduce a good xml.
        
        ... And yeah, I hate XML with unit testing :("""
        
        xml_string = """<?xml version="1.0" ?>
<ncx version="2005-1" xml:lang="en-US" xmlns="http://www.daisy.org/z3986/2005/ncx/">
    <head>
        <meta content="org-example-5059463624137734586" name="dtb:uid"/>
        <meta content="2" name="dtb:depth"/>
        <meta content="3907" name="dtb:totalPageCount"/>
        <meta content="7814" name="dtb:maxPageNumber"/>
        <meta content="Epub Test" name="dtb:generator"/>
    </head>
    <docTitle>
        <text>Selections from &quot;Great Pictures, As Seen and Described by Famous Writers&quot;</text>
    </docTitle>
    <docAuthor>
        <text>Esther Singleton</text>
    </docAuthor>
    <docAuthor>
        <text>Test Author</text>
    </docAuthor>
    <navMap>
        <navPoint class="h1" id="ch1">
            <navLabel>
                <text>Chapter 1</text>
            </navLabel>
            <content src="content.html#ch_1"/>
            <navPoint class="h2" id="ch_1_1">
                <navLabel>
                    <text>Chapter 1.1</text>
                </navLabel>
                <content src="content.html#ch_1_1"/>
            </navPoint>
        </navPoint>
        <navPoint class="h1" id="ncx-2">
            <navLabel>
                <text>Chapter 2</text>
            </navLabel>
            <content src="content.html#ch_2"/>
        </navPoint>
    </navMap>
    <pageList>
        <pageTarget id="p1" type="normal" value="1">
            <navLabel dir="ltr">
                <text>1</text>
            </navLabel>
            <content src="content.html#p1"/>
        </pageTarget>
        <pageTarget id="p2" type="normal" value="2">
            <navLabel>
                <text>2</text>
            </navLabel>
            <content src="content.html#p2"/>
        </pageTarget>
    </pageList>
    <navList>
        <navLabel>
            <text>List of Illustrations</text>
        </navLabel>
        <navTarget id="ill-1">
            <navLabel>
                <text>Portratit of Georg Gisze (Holbein)</text>
            </navLabel>
            <content src="content.html#ill1"/>
        </navTarget>
        <navTarget id="ill-2">
            <navLabel>
                <text>The adoration of the lamb (Van Eyck)</text>
            </navLabel>
            <content src="content.html#ill2"/>
        </navTarget>
    </navList>
</ncx>
"""

        self.maxDiff = None

        toc = epub.ncx.parse_toc(xml_string)
        xml_toc = toc.as_xml_document().toprettyxml('    ')

        xml_input = xml_string.strip()
        xml_output = xml_toc.strip()

        self.assertEqual(xml_output, xml_input)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
Python lib for reading OPF formated file for epub.

Since the "Tour" element is deprecated in Epub 2, it is not supported by this
library.

OPF epub : http://idpf.org/epub/20/spec/OPF_2.0.1_draft.htm
"""


from xml.dom import minidom


try:
    # Only for Python 2.7+
    from collections import OrderedDict
except ImportError:
    try:
        # For Python 2.6
        from ordereddict import OrderedDict
    except ImportError:
        raise ImportError(
            'You should use Python 2.7 or install `ordereddict` from pypi.')


from epub.utils import get_node_text


XMLNS_DC = 'http://purl.org/dc/elements/1.1/'
XMLNS_OPF = 'http://www.idpf.org/2007/opf'


def parse_opf(xml_string):
    package = minidom.parseString(xml_string).documentElement

    # Get Uid
    uid_id = package.getAttribute('unique-identifier')

    # Store each child nodes into a dict (metadata, manifest, spine, guide)
    data = {'metadata': None,
            'manifest': None,
            'spine': None,
            'guide': None}
    elements = [e for e in package.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in elements:
        data[node.tagName.lower()] = node

    # Inspect metadata
    metadata = _parse_xml_metadata(data['metadata'])

    # Inspect manifest
    manifest = _parse_xml_manifest(data['manifest'])

    # Inspect spine
    spine = _parse_xml_spine(data['spine'])

    # Inspect guide if exist
    if data['guide'] is None:
        guide = None
    else:
        guide = _parse_xml_guide(data['guide'])

    opf = Opf(uid_id=uid_id,
              metadata=metadata,
              manifest=manifest,
              spine=spine,
              guide=guide)
    return opf


def _parse_xml_metadata(element):
    """Extract metadata from an xml.dom.Element object (ELEMENT_NODE)

    The "<metadata>" tag has a lot of metadatas about the epub this method
    inspect and store into object attributes (like "title" or "creator").
    """
    metadata = Metadata()

    for node in element.getElementsByTagName('dc:title'):
        metadata.add_title(get_node_text(node),
                           node.getAttribute('xml:lang'))

    for node in element.getElementsByTagName('dc:creator'):
        metadata.add_creator(get_node_text(node),
                             node.getAttribute('opf:role'),
                             node.getAttribute('opf:file-as'))

    for node in element.getElementsByTagName('dc:subject'):
        metadata.add_subject(get_node_text(node))

    for node in element.getElementsByTagName('dc:description'):
        metadata.description = get_node_text(node)

    for node in element.getElementsByTagName('dc:publisher'):
        metadata.publisher = get_node_text(node)

    for node in element.getElementsByTagName('dc:contributor'):
        metadata.add_contributor(get_node_text(node),
                                 node.getAttribute('opf:role'),
                                 node.getAttribute('opf:file-as'))

    for node in element.getElementsByTagName('dc:date'):
        metadata.add_date(get_node_text(node),
                          node.getAttribute('opf:event'))

    for node in element.getElementsByTagName('dc:type'):
        metadata.dc_type = get_node_text(node)

    for node in element.getElementsByTagName('dc:format'):
        metadata.format = get_node_text(node)

    for node in element.getElementsByTagName('dc:identifier'):
        metadata.add_identifier(get_node_text(node),
                            node.getAttribute('id'),
                            node.getAttribute('opf:scheme'))

    for node in element.getElementsByTagName('dc:source'):
        metadata.source = get_node_text(node)

    for node in element.getElementsByTagName('dc:language'):
        metadata.add_language(get_node_text(node))

    for node in element.getElementsByTagName('dc:relation'):
        metadata.relation = get_node_text(node)

    for node in element.getElementsByTagName('dc:coverage'):
        metadata.coverage = get_node_text(node)

    for node in element.getElementsByTagName('dc:rights'):
        metadata.right = get_node_text(node)

    for node in element.getElementsByTagName('meta'):
        metadata.add_meta(node.getAttribute('name'),
                          node.getAttribute('content'))

    return metadata


def _parse_xml_manifest(element):
    """Inspect an xml.dom.Element <manifest> and return a list of
    epub.EpubManifestItem object."""

    manifest = Manifest()
    for e in element.getElementsByTagName('item'):
        manifest.add_item(e.getAttribute('id'),
                          e.getAttribute('href'),
                          e.getAttribute('media-type'),
                          e.getAttribute('fallback'),
                          e.getAttribute('required-namespace'),
                          e.getAttribute('required-modules'),
                          e.getAttribute('fallback-style'))
    return manifest


def _parse_xml_spine(element):
    """Inspect an xml.dom.Element <spine> and return epub.opf.Spine object"""

    spine = Spine()
    spine.toc = element.getAttribute('toc')
    for e in element.getElementsByTagName('itemref'):
        spine.add_itemref(e.getAttribute('idref'),
                          e.getAttribute('linear').lower() != 'no')
    return spine


def _parse_xml_guide(element):
    """Inspect an xml.dom.Element <guide> and return a list of ref as tuple."""

    guide = Guide()
    for e in element.getElementsByTagName('reference'):
        guide.add_reference(e.getAttribute('href'),
                            e.getAttribute('type'),
                            e.getAttribute('title'))
    return guide


class Opf(object):
    """Represent an OPF formated file.

    OPF is an xml formated file, used in the epub spec."""

    def __init__(self, uid_id=None, version=None, xmlns=None,
                 metadata=None, manifest=None, spine=None, guide=None):
        self.uid_id = uid_id
        self.version = version if version else '2.0'
        self.xmlns = xmlns if xmlns else XMLNS_OPF

        if metadata is None:
            self.metadata = Metadata()
        else:
            self.metadata = metadata
        if manifest is None:
            self.manifest = Manifest()
        else:
            self.manifest = manifest
        if spine is None:
            self.spine = Spine()
        else:
            self.spine = spine
        if guide is None:
            self.guide = Guide()
        else:
            self.guide = guide

    def as_xml_document(self):
        doc = minidom.Document()
        package = doc.createElement('package')
        package.setAttribute('version', self.version)
        package.setAttribute('unique-identifier', self.uid_id)
        package.setAttribute('xmlns', self.xmlns)
        package.appendChild(self.metadata.as_xml_element())
        package.appendChild(self.manifest.as_xml_element())
        package.appendChild(self.spine.as_xml_element())
        package.appendChild(self.guide.as_xml_element())
        doc.appendChild(package)
        return doc


class Metadata(object):
    """Represent an epub's metadatas set.

    See http://idpf.org/epub/20/spec/OPF_2.0.1_draft.htm#Section2.2"""

    def __init__(self):
        self.titles = []
        self.creators = []
        self.subjects = []
        self.description = None
        self.publisher = None
        self.contributors = []
        self.dates = []
        self.dc_type = None
        self.format = None
        self.identifiers = []
        self.source = None
        self.languages = []
        self.relation = None
        self.coverage = None
        self.right = None
        self.metas = []

    def add_title(self, title, lang=None):
        lang = lang or ''
        self.titles.append((title, lang))

    def add_creator(self, name, role=None, file_as=None):
        role = role or ''
        file_as = file_as or ''
        self.creators.append((name, role, file_as))

    def add_subject(self, subject):
        self.subjects.append(subject)

    def add_contributor(self, name, role=None, file_as=None):
        role = role or ''
        file_as = file_as or ''
        self.contributors.append((name, role, file_as))

    def add_date(self, date, event=None):
        event = event or ''
        self.dates.append((date, event))

    def add_identifier(self, content, identifier=None, scheme=None):
        identifier = identifier or ''
        scheme = scheme or ''
        self.identifiers.append((content, identifier, scheme))

    def add_language(self, lang):
        self.languages.append(lang)

    def add_meta(self, name, content):
        self.metas.append((name, content))

    def get_isbn(self):
        l = [x[0] for x in self.identifiers if x[2].lower() == 'isbn']
        isbn = None
        if l:
            isbn = l[0]
        return isbn

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        metadata = doc.createElement('metadata')
        metadata.setAttribute('xmlns:dc', XMLNS_DC)
        metadata.setAttribute('xmlns:opf', XMLNS_OPF)

        for text, lang in self.titles:
            title = doc.createElement('dc:title')
            if lang:
                title.setAttribute('xml:lang', lang)
            title.appendChild(doc.createTextNode(text))
            metadata.appendChild(title)

        for name, role, file_as in self.creators:
            creator = doc.createElement('dc:creator')
            if role:
                creator.setAttribute('opf:role', role)
            if file_as:
                creator.setAttribute('opf:file-as', file_as)
            creator.appendChild(doc.createTextNode(name))
            metadata.appendChild(creator)

        for text in self.subjects:
            subject = doc.createElement('dc:subject')
            subject.appendChild(doc.createTextNode(text))
            metadata.appendChild(subject)

        if self.description:
            description = doc.createElement('dc:description')
            description.appendChild(doc.createTextNode(self.description))
            metadata.appendChild(description)

        if self.publisher:
            publisher = doc.createElement('dc:publisher')
            publisher.appendChild(doc.createTextNode(self.publisher))
            metadata.appendChild(publisher)

        for name, role, file_as in self.contributors:
            contributor = doc.createElement('dc:contributor')
            if role:
                contributor.setAttribute('opf:role', role)
            if file_as:
                contributor.setAttribute('opf:file-as', file_as)
            contributor.appendChild(doc.createTextNode(name))
            metadata.appendChild(contributor)

        for text, event in self.dates:
            date = doc.createElement('dc:date')
            if event:
                date.setAttribute('opf:event', event)
            date.appendChild(doc.createTextNode(text))
            metadata.appendChild(date)

        if self.dc_type:
            dc_type = doc.createElement('dc:type')
            dc_type.appendChild(doc.createTextNode(self.dc_type))
            metadata.appendChild(dc_type)

        if self.format:
            dc_format = doc.createElement('dc:format')
            dc_format.appendChild(doc.createTextNode(self.format))
            metadata.appendChild(dc_format)

        for text, identifier, scheme in self.identifiers:
            dc_identifier = doc.createElement('dc:identifier')
            if identifier:
                dc_identifier.setAttribute('id', identifier)
            if scheme:
                dc_identifier.setAttribute('opf:scheme', scheme)
            dc_identifier.appendChild(doc.createTextNode(text))
            metadata.appendChild(dc_identifier)

        if self.source:
            source = doc.createElement('dc:source')
            source.appendChild(doc.createTextNode(self.source))
            metadata.appendChild(source)

        for text in self.languages:
            language = doc.createElement('dc:language')
            language.appendChild(doc.createTextNode(text))
            metadata.appendChild(language)

        if self.relation:
            relation = doc.createElement('dc:relation')
            relation.appendChild(doc.createTextNode(self.relation))
            metadata.appendChild(relation)

        if self.coverage:
            coverage = doc.createElement('dc:coverage')
            coverage.appendChild(doc.createTextNode(self.coverage))
            metadata.appendChild(coverage)

        if self.right:
            right = doc.createElement('dc:rights')
            right.appendChild(doc.createTextNode(self.right))
            metadata.appendChild(right)

        for name, content in self.metas:
            meta = doc.createElement('meta')
            meta.setAttribute('name', name)
            meta.setAttribute('content', content)
            metadata.appendChild(meta)

        return metadata


class Manifest(OrderedDict):

    def __contains__(self, item):
        if hasattr(item, 'identifier'):
            return super(Manifest, self).__contains__(item.identifier)
        else:
            return super(Manifest, self).__contains__(item)

    def __setitem__(self, key, value):
        if hasattr(value, 'identifier') and hasattr(value, 'href'):
            if value.identifier == key:
                super(Manifest, self).__setitem__(key, value)
            else:
                raise ValueError('Value\'s id is different from insert key.')
        else:
            requierements = 'id and href attributes'
            msg = 'Value does not fit the requirement (%s).' % requierements
            raise ValueError(msg)

    def add_item(self, identifier, href, media_type=None, fallback=None,
                 required_namespace=None, required_modules=None,
                 fallback_style=None):
        item = ManifestItem(identifier, href, media_type,
                            fallback, required_namespace, required_modules,
                            fallback_style)
        self.append(item)

    def append(self, item):
        if hasattr(item, 'identifier') and \
           hasattr(item, 'href') and \
           hasattr(item, 'as_xml_element'):
            self.__setitem__(item.identifier, item)
        else:
            raise ValueError('Manifest item must have [identifier, href, ' + \
                             'as_xml_element()] attributes and method.')

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        manifest = doc.createElement('manifest')

        for item in self.values():
            manifest.appendChild(item.as_xml_element())

        return manifest


class ManifestItem(object):
    """Represent an item from the epub's manifest."""

    def __init__(self, identifier, href, media_type=None, fallback=None,
                 required_namespace=None, required_modules=None,
                 fallback_style=None):
        self.identifier = identifier
        self.href = href
        self.media_type = media_type
        self.fallback = fallback
        self.required_namespace = required_namespace
        self.required_modules = required_modules
        self.fallback_style = fallback_style

    def as_xml_element(self):
        """Return an xml dom Element node."""

        item = minidom.Document().createElement("item")

        item.setAttribute('id', self.identifier)
        item.setAttribute('href', self.href)
        if self.media_type:
            item.setAttribute('media-type', self.media_type)
        if self.fallback:
            item.setAttribute('fallback', self.fallback)
        if self.required_namespace:
            item.setAttribute('required-namespace', self.required_namespace)
        if self.required_modules:
            item.setAttribute('required-modules', self.required_modules)
        if self.fallback_style:
            item.setAttribute('fallback-style', self.fallback_style)

        return item


class Spine(object):

    def __init__(self, toc=None, itemrefs=None):
        self.toc = toc
        if itemrefs is None:
            self.itemrefs = []
        else:
            self.itemrefs = itemrefs

    def add_itemref(self, idref, linear=True):
        self.append((idref, linear))

    def append(self, itemref):
        self.itemrefs.append(itemref)

    def as_xml_element(self):
        doc = minidom.Document()
        spine = doc.createElement('spine')
        spine.setAttribute('toc', self.toc)

        for idref, linear in self.itemrefs:
            itemref = doc.createElement('itemref')
            itemref.setAttribute('idref', idref)
            if not linear:
                itemref.setAttribute('linear', 'no')
            spine.appendChild(itemref)

        return spine


class Guide(object):

    def __init__(self):
        self.references = []

    def add_reference(self, href, ref_type=None, title=None):
        self.append((href, ref_type, title))

    def append(self, reference):
        self.references.append(reference)

    def as_xml_element(self):
        doc = minidom.Document()
        guide = doc.createElement('guide')

        for href, ref_type, title in self.references:
            reference = doc.createElement('reference')
            if type:
                reference.setAttribute('type', ref_type)
            if title:
                reference.setAttribute('title', title)
            if href:
                reference.setAttribute('href', href)
            guide.appendChild(reference)

        return guide



# -*- coding: utf-8 -*-
from distutils.core import setup


with open("README.txt") as f:
    long_description = f.read()


setup(name='epub',
      version='0.5.0',
      description='Library to open and read files in the epub version 2.',
      long_description=long_description,
      author='Florian Strzelecki',
      author_email='florian.strzelecki@gmail.com',
      license='LGPL',
      url='http://epub.exirel.me',
      packages=['epub'],
      classifiers=['Development Status :: 4 - Beta',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
                   'Programming Language :: Python :: 2.7',
                   'Programming Language :: Python :: 3.2',
                   'Topic :: Software Development :: Libraries :: Python Modules'])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
Python lib for reading NCX formated file for epub.

There is some difference between NCX original format and one for Epub; see
officiel documention for more information.1111

NCX doc: http://www.niso.org/workrooms/daisy/Z39-86-2005.html#NCX
NCX Epub spec: http://idpf.org/epub/20/spec/OPF_2.0.1_draft.htm#Section2.4.1
"""


from xml.dom import minidom


def parse_toc(xmlstring):
    """Inspect an NCX formated xml document."""
    toc = Ncx()
    toc_xml = minidom.parseString(xmlstring).documentElement

    xmlns = toc_xml.getAttribute('xmlns')
    if xmlns:
        toc.xmlns = xmlns

    version = toc_xml.getAttribute('version')
    if version:
        toc.version = version

    lang = toc_xml.getAttribute('xml:lang')
    if lang:
        toc.lang = lang

    # Inspect head > meta; unknow meta are ignored
    head = toc_xml.getElementsByTagName('head')[0]
    metas = {'dtb:uid': '',
             'dtb:depth': '',
             'dtb:totalPageCount': '',
             'dtb:maxPageNumber': '',
             'dtb:generator': ''}

    for meta in head.getElementsByTagName('meta'):
        metas[meta.getAttribute('name')] = meta.getAttribute('content')

    toc.uid = metas['dtb:uid']
    toc.depth = metas['dtb:depth']
    toc.total_page_count = metas['dtb:totalPageCount']
    toc.max_page_number = metas['dtb:maxPageNumber']
    toc.generator = metas['dtb:generator']

    # Get title (one and only one <docTitle> tag is required)
    doc_title_node = toc_xml.getElementsByTagName('docTitle')[0]
    toc.title = _parse_for_text_tag(doc_title_node)

    # Get authors (<docAuthor> tags are optionnal)
    for author in toc_xml.getElementsByTagName('docAuthor'):
        toc.authors.append(_parse_for_text_tag(author))

    # Inspect <navMap> (one is required)
    nav_map_node = toc_xml.getElementsByTagName('navMap')[0]
    toc.nav_map = _parse_xml_nav_map(nav_map_node)

    # Inspect <pageList> (optionnal, only one)
    page_lists = toc_xml.getElementsByTagName('pageList')
    if len(page_lists) > 0:
        toc.page_list = _parse_xml_page_list(page_lists[0])

    # Inspect <navList> (optionnal, many are possible)
    for nav_list in toc_xml.getElementsByTagName('navList'):
        toc.add_nav_list(_parse_xml_nav_list(nav_list))

    return toc


def _parse_xml_nav_map(element):
    """Inspect an xml.dom.Element <navMap> and return a NcxNavMap object."""
    nav_map = NavMap()
    nav_map.identifier = element.getAttribute('id')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            nav_map.add_label(_parse_for_text_tag(node),
                              node.getAttribute('xml:lang'),
                              node.getAttribute('dir'))
        elif node.tagName == 'navInfo':
            nav_map.add_info(_parse_for_text_tag(node),
                             node.getAttribute('xml:lang'),
                             node.getAttribute('dir'))
        elif node.tagName == 'navPoint':
            nav_map.add_point(_parse_xml_nav_point(node))

    return nav_map


def _parse_xml_nav_point(element):
    """Inspect an xml.dom.Element <navPoint> and return a NcxNavPoint object.
    """
    nav_point = NavPoint()
    nav_point.identifier = element.getAttribute('id')
    nav_point.class_name = element.getAttribute('class')
    nav_point.play_order = element.getAttribute('playOrder')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            nav_point.add_label(_parse_for_text_tag(node),
                                node.getAttribute('xml:lang'),
                                node.getAttribute('dir'))
        elif node.tagName == 'content':
            nav_point.src = node.getAttribute('src')
        elif node.tagName == 'navPoint':
            nav_point.add_point(_parse_xml_nav_point(node))

    return nav_point


def _parse_xml_page_list(element):
    """Inspect an xml.dom.Element <pageList> and return a NcxPageList object.
    """
    page_list = PageList()
    page_list.identifier = element.getAttribute('id')
    page_list.class_name = element.getAttribute('class')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            page_list.add_label(_parse_for_text_tag(node),
                                node.getAttribute('xml:lang'),
                                node.getAttribute('dir'))
        elif node.tagName == 'navInfo':
            page_list.add_info(_parse_for_text_tag(node),
                               node.getAttribute('xml:lang'),
                               node.getAttribute('dir'))
        elif node.tagName == 'pageTarget':
            page_list.add_target(_parse_xml_page_target(node))

    return page_list


def _parse_xml_page_target(element):
    """Inspect an xml.dom.Element <pageTarget> and return a NcxPageTarget
    object."""
    page_target = PageTarget()
    page_target.identifier = element.getAttribute('id')
    page_target.value = element.getAttribute('value')
    page_target.target_type = element.getAttribute('type')
    page_target.class_name = element.getAttribute('class')
    page_target.play_order = element.getAttribute('playOrder')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            page_target.add_label(_parse_for_text_tag(node),
                                  node.getAttribute('xml:lang'),
                                  node.getAttribute('dir'))
        elif node.tagName == 'content':
            page_target.src = node.getAttribute('src')

    return page_target


def _parse_xml_nav_list(element):
    """Inspect an xml.dom.Element <navList> and return a NcxNavList object."""
    nav_list = NavList()
    nav_list.identifier = element.getAttribute('id')
    nav_list.class_name = element.getAttribute('class')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            nav_list.add_label(_parse_for_text_tag(node),
                                node.getAttribute('xml:lang'),
                                node.getAttribute('dir'))
        elif node.tagName == 'navInfo':
            nav_list.add_info(_parse_for_text_tag(node),
                               node.getAttribute('xml:lang'),
                               node.getAttribute('dir'))
        elif node.tagName == 'navTarget':
            nav_list.add_target(_parse_xml_nav_target(node))

    return nav_list


def _parse_xml_nav_target(element):
    """Inspect an xml.dom.Element <navTarget> and return a NcxNavTarget
    object."""
    nav_target = NavTarget()
    nav_target.identifier = element.getAttribute('id')
    nav_target.value = element.getAttribute('value')
    nav_target.class_name = element.getAttribute('class')
    nav_target.play_order = element.getAttribute('playOrder')

    children = [e for e in element.childNodes if e.nodeType == e.ELEMENT_NODE]
    for node in children:
        if node.tagName == 'navLabel':
            nav_target.add_label(_parse_for_text_tag(node),
                                  node.getAttribute('xml:lang'),
                                  node.getAttribute('dir'))
        elif node.tagName == 'content':
            nav_target.src = node.getAttribute('src')

    return nav_target


def _parse_for_text_tag(xml_element, name=None):
    """Inspect an xml.dom.Element with a child 'name' to get its text value.

    NCX file has many element with a child likes
    "navLabel" > "text" > TEXT_NODE
    and this function allow to avoid some boilerplate code.

    First parameter must be an xml.dom.Element, having one child named by the
    second parameter (by default a "text" tag).

    If nothing is founded, an empty string '' is returned.

    Whitespaces and tabulations are stripped."""
    name = name or 'text'
    tags = [e for e in xml_element.childNodes
              if e.nodeType == e.ELEMENT_NODE and e.tagName == name]
    text = ''
    if len(tags) > 0:
        tag = tags[0]
        if tag.firstChild and tag.firstChild.data:
            tag.normalize()
            text = tag.firstChild.data.strip()
    return text


def _create_xml_element_text(data, name=None):
    """Create a <text> ... </text> Element node.

    You can use a different tag name with the name argument
    (default is "text").

    If data is None or empty, it will create an empty element tag, eg. :
    <emptyTag/> instead of <emptyTag></emptyTag>"""
    if name is None:
        name = 'text'
    doc = minidom.Document()
    element = doc.createElement(name)
    if data:
        element.appendChild(doc.createTextNode(data))
    return element


class Ncx(object):
    """Represent the structured content of a NCX file."""

    def __init__(self, nav_map=None, page_list=None):
        self.xmlns = 'http://www.daisy.org/z3986/2005/ncx/'
        self.version = '2005-1'
        self.lang = None
        self.uid = None
        self.depth = None
        self.total_page_count = None
        self.max_page_number = None
        self.generator = None
        self.title = None
        self.authors = []
        if nav_map is None:
            nav_map = NavMap()
        self.nav_map = nav_map
        if page_list is None:
            page_list = PageList()
        self.page_list = page_list
        self.nav_lists = []

    def add_nav_list(self, nav_list):
        self.nav_lists.append(nav_list)

    def as_xml_document(self):
        """Return an xml dom Document node."""
        doc = minidom.Document()
        ncx = doc.createElement('ncx')
        ncx.setAttribute('xmlns', self.xmlns)
        ncx.setAttribute('version', self.version)
        if self.lang:
            ncx.setAttribute('xml:lang', self.lang)

        # head
        ncx.appendChild(self._head_as_xml_element())

        # title
        title = doc.createElement('docTitle')
        title.appendChild(_create_xml_element_text(self.title))
        ncx.appendChild(title)

        # authors
        for text in self.authors:
            author = doc.createElement('docAuthor')
            author.appendChild(_create_xml_element_text(text))
            ncx.appendChild(author)

        # nav_map
        ncx.appendChild(self.nav_map.as_xml_element())

        # page_list
        if self.page_list:
            ncx.appendChild(self.page_list.as_xml_element())

        # nav_lists
        for nav_list in self.nav_lists:
            ncx.appendChild(nav_list.as_xml_element())

        doc.appendChild(ncx)
        return doc

    def _head_as_xml_element(self):
        """Create an xml Element node <head> with meta-data of Ncx item."""
        doc = minidom.Document()
        head = doc.createElement('head')
        if self.uid:
            head.appendChild(self._meta_as_xml_element('dtb:uid', self.uid))
        if self.depth:
            head.appendChild(self._meta_as_xml_element('dtb:depth',
                                                       self.depth))
        if self.total_page_count:
            head.appendChild(self._meta_as_xml_element('dtb:totalPageCount',
                                                       self.total_page_count))
        if self.max_page_number:
            head.appendChild(self._meta_as_xml_element('dtb:maxPageNumber',
                                                       self.max_page_number))
        if self.generator:
            head.appendChild(self._meta_as_xml_element('dtb:generator',
                                                       self.generator))
        return head

    def _meta_as_xml_element(self, name, content):
        """Create an xml Element node <meta> with attributes name & content."""
        doc = minidom.Document()
        meta = doc.createElement('meta')
        meta.setAttribute('name', name)
        meta.setAttribute('content', content)
        return meta


class NavMap(object):
    """Represente navMap tag of an NCX file."""

    def __init__(self):
        self.identifier = None
        self.labels = []
        self.infos = []
        self.nav_point = []

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def add_info(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.infos.append((label, lang, direction))

    def add_point(self, point):
        self.nav_point.append(point)

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        nav_map = doc.createElement('navMap')

        if self.identifier:
            nav_map.setAttribute('id', self.identifier)

        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            nav_map.appendChild(label)

        for text, lang, direction in self.infos:
            info = doc.createElement('navInfo')
            info.appendChild(_create_xml_element_text(text))
            if lang:
                info.setAttribute('xml:lang', lang)
            if direction:
                info.setAttribute('dir', direction)
            nav_map.appendChild(info)

        for nav_point in self.nav_point:
            nav_map.appendChild(nav_point.as_xml_element())

        return nav_map


class NavPoint(object):

    def __init__(self):
        self.identifier = None
        self.class_name = None
        self.play_order = None
        self.labels = []
        self.src = None
        self.nav_point = []

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def add_point(self, nav_point):
        self.nav_point.append(nav_point)

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        nav_point = doc.createElement('navPoint')

        # Attributes
        if self.identifier:
            nav_point.setAttribute('id', self.identifier)

        if self.class_name:
            nav_point.setAttribute('class', self.class_name)

        if self.play_order:
            nav_point.setAttribute('playOrder', self.play_order)

        # navLabel
        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            nav_point.appendChild(label)

        # content
        content = doc.createElement('content')
        content.setAttribute('src', self.src)
        nav_point.appendChild(content)

        # navPoint
        for child in self.nav_point:
            nav_point.appendChild(child.as_xml_element())

        return nav_point


class PageList(object):

    def __init__(self):
        self.identifier = None
        self.class_name = None
        self.page_target = []
        self.labels = []
        self.infos = []

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def add_info(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.infos.append((label, lang, direction))

    def add_target(self, page_target):
        self.page_target.append(page_target)

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        page_list = doc.createElement('pageList')

        # attributes
        if self.identifier:
            page_list.setAttribute('id', self.identifier)

        if self.class_name:
            page_list.setAttribute('class', self.class_name)

        # navLabel
        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            page_list.appendChild(label)

        # navInfo
        for text, lang, direction in self.infos:
            info = doc.createElement('navInfo')
            info.appendChild(_create_xml_element_text(text))
            if lang:
                info.setAttribute('xml:lang', lang)
            if direction:
                info.setAttribute('dir', direction)
            page_list.appendChild(info)

        # pageTarget
        for child in self.page_target:
            page_list.appendChild(child.as_xml_element())

        return page_list


class PageTarget(object):

    def __init__(self):
        self.identifier = None
        self.value = None
        self.target_type = None
        self.class_name = None
        self.play_order = None
        self.src = None
        self.labels = []

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        page_target = doc.createElement('pageTarget')

        # attributes
        if self.identifier:
            page_target.setAttribute('id', self.identifier)

        if self.value:
            page_target.setAttribute('value', self.value)

        if self.target_type:
            page_target.setAttribute('type', self.target_type)

        if self.class_name:
            page_target.setAttribute('class', self.class_name)

        if self.play_order:
            page_target.setAttribute('playOrder', self.play_order)

        # navLabel
        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            page_target.appendChild(label)

        # content
        content = doc.createElement('content')
        content.setAttribute('src', self.src)
        page_target.appendChild(content)

        return page_target


class NavList(object):

    def __init__(self):
        self.identifier = None
        self.class_name = None
        self.nav_target = []
        self.labels = []
        self.infos = []

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def add_info(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.infos.append((label, lang, direction))

    def add_target(self, nav_target):
        self.nav_target.append(nav_target)

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        nav_list = doc.createElement('navList')

        # attributes
        if self.identifier:
            nav_list.setAttribute('id', self.identifier)

        if self.class_name:
            nav_list.setAttribute('class', self.class_name)

        # navLabel
        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            nav_list.appendChild(label)

        # navInfo
        for text, lang, direction in self.infos:
            info = doc.createElement('navInfo')
            info.appendChild(_create_xml_element_text(text))
            if lang:
                info.setAttribute('xml:lang', lang)
            if direction:
                info.setAttribute('dir', direction)
            nav_list.appendChild(info)

        # navTarget
        for nav_target in self.nav_target:
            nav_list.appendChild(nav_target.as_xml_element())

        return nav_list


class NavTarget(object):

    def __init__(self):
        self.identifier = None
        self.class_name = None
        self.value = None
        self.play_order = None
        self.labels = []
        self.src = None

    def add_label(self, label, lang=None, direction=None):
        lang = lang or ''
        direction = direction or ''
        self.labels.append((label, lang, direction))

    def as_xml_element(self):
        """Return an xml dom Element node."""
        doc = minidom.Document()
        nav_target = doc.createElement('navTarget')

        # attributes
        if self.identifier:
            nav_target.setAttribute('id', self.identifier)

        if self.class_name:
            nav_target.setAttribute('class', self.class_name)

        if self.value:
            nav_target.setAttribute('value', self.value)

        if self.play_order:
            nav_target.setAttribute('playOrder', self.play_order)

        # navLabel
        for text, lang, direction in self.labels:
            label = doc.createElement('navLabel')
            label.appendChild(_create_xml_element_text(text))
            if lang:
                label.setAttribute('xml:lang', lang)
            if direction:
                label.setAttribute('dir', direction)
            nav_target.appendChild(label)

        # content
        content = doc.createElement('content')
        content.setAttribute('src', self.src)
        nav_target.appendChild(content)

        return nav_target
